from typing import Tuple, List, Union
from sqlexecutorx import page_select, page_query

from .loader import Loader
from .dialect import Dialect
from .constant import SELECT_COUNT
from .table_limit_exec import LimitExec, WhereLimitExec
from .table_support import get_table_select_sql, get_where_arg_limit


class TablePageExec:

    def __init__(self, table_exec, page_num: int, page_size: int, return_total: bool):
        self.table_exec = table_exec
        self.page_num = page_num
        self.page_size = page_size
        self.return_total = return_total

    def select(self, *columns) -> List[Tuple]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').page(1, 10).select('id', 'name', 'age')
        [(3, '张三', 20)]
        """
        sql = get_table_select_sql(self.table_exec.table, None, None, None, *columns)
        if self.return_total:
            count_sql = get_table_select_sql(self.table_exec.table, None, None, None, SELECT_COUNT)
            sql, args = Dialect.get_page_sql_args(sql, self.page_num, self.page_size)
            return page_select(count_sql, sql, *args)
        return self.table_exec.exec.do_select_page(sql, self.page_num, self.page_size)

    def query(self, *columns) -> List[dict]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').page(1, 10).query('id', 'name', 'age')
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        sql = get_table_select_sql(self.table_exec.table, None, None, None, *columns)
        if self.return_total:
            count_sql = get_table_select_sql(self.table_exec.table, None, None, None, SELECT_COUNT)
            sql, args = Dialect.get_page_sql_args(sql, self.page_num, self.page_size)
            return page_query(count_sql, sql, *args)
        return self.table_exec.exec.do_query_page(sql, self.page_num, self.page_size)

    def load(self, *columns) -> Loader:
        """
        Select page data from table and return a Loader instance

        :return: Loader

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').page(1, 10).load('id', 'name', 'age')
        Lodder()
        """
        sql = get_table_select_sql(self.table_exec.table, None, None, None, *columns)
        sql, args = Dialect.get_page_sql_args(sql, self.page_num, self.page_size)
        return self.table_exec.exec.do_load(sql, *args)


class ColumnPageExec:

    def __init__(self, table_page_exec: TablePageExec, *columns):
        self.table_page_exec = table_page_exec
        self.columns = columns

    def select(self) -> List[Tuple]:
        """
        Select data from table and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').page(1, 10).select()
        [(3, '张三', 20)]
        """
        return self.table_page_exec.select(*self.columns)

    def query(self) -> List[dict]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').page(1, 10).query()
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        return self.table_page_exec.query(*self.columns)

    def to_df(self):
        """
        Select from table and return pandas DataFrame instance.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').page(1, 10).to_df()
        """
        return self.table_page_exec.load(*self.columns).to_df()

    def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
        """
        Select from table and sava as a csv file.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').page(1, 10).to_csv('test.csv')
        """
        self.table_page_exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)

    def to_json(self, file_name: str, encoding='utf-8'):
        """
        Select from table and sava as a json file.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').page(1, 10).to_json('test.json')
        """
        self.table_page_exec.load(*self.columns).to_json(file_name, encoding)


class WherePageExec:

    def __init__(self, where_exec, page_num, page_size, return_total: bool):
        self.where_exec = where_exec
        self.page_num = page_num
        self.page_size = page_size
        self.return_total = return_total

    def select(self, *columns) -> Union[List[Tuple], Tuple[int, List[Tuple]]]:
        """
        Select data from table and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').where(name='张三', age=20).page(1, 10).select('id', 'name', 'age')
        [(3, '张三', 20)]
        """
        sql, args = self.where_exec.get_select_sql_args(*columns)
        if self.return_total:
            count_sql, _ = self.where_exec.get_select_sql_args(SELECT_COUNT)
            sql, args = Dialect.get_page_sql_args(sql, self.page_num, self.page_size, *args)
            return page_select(count_sql, sql, *args)
        return self.where_exec.exec.do_select_page(sql, self.page_num, self.page_size, *args)

    def query(self, *columns) -> Union[List[dict], Tuple[int, List[dict]]]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').where(name='张三', age=20).page(1, 10).query('id', 'name', 'age')
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        sql, args = self.where_exec.get_select_sql_args(*columns)
        if self.return_total:
            count_sql, _ = self.where_exec.get_select_sql_args(SELECT_COUNT)
            sql, args = Dialect.get_page_sql_args(sql, self.page_num, self.page_size, *args)
            return page_query(count_sql, sql, *args)
        return self.where_exec.exec.do_query_page(sql, self.page_num, self.page_size, *args)

    def load(self, *columns) -> Loader:
        """
        Select page data from table and return a Loader instance

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').where(name='张三', age=20).page(1, 10).load('id', 'name', 'age')
        Loader()
        """
        sql, args = self.where_exec.get_select_sql_args(*columns)
        sql, args = Dialect.get_page_sql_args(sql, self.page_num, self.page_size, *args)
        return self.where_exec.exec.do_load(sql, *args)


class ColumnWherePageExec:

    def __init__(self, where_page_exec: WherePageExec, *columns):
        self.where_page_exec = where_page_exec
        self.columns = columns

    def select(self) -> List[Tuple]:
        """
        Select data from table and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).page(1, 10).select()
        [(3, '张三', 20)]
        """
        return self.where_page_exec.select(*self.columns)

    def query(self) -> List[dict]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).page(1, 10).query()
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        return self.where_page_exec.query(*self.columns)

    def to_df(self):
        """
        Select from table and return pandas DataFrame instance.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).page(1, 10).to_df()
        """
        return self.where_page_exec.load(*self.columns).to_df()

    def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
        """
        Select from table and sava as a csv file.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).page(1, 10).to_csv('test.csv')
        """
        self.where_page_exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)

    def to_json(self, file_name: str, encoding='utf-8'):
        """
        Select from table and sava as a json file.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).page(1, 10).to_json('test.json')
        """
        self.where_page_exec.load(*self.columns).to_json(file_name, encoding)
        
        
class OrderByPageExec:
    
    def __init__(self, limit_exec: LimitExec, return_total: bool):
        self.limit_exec = limit_exec
        self.return_total = return_total
        
    def select(self, *columns) -> List[Tuple]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').order_by('id DESC, name ASC').page(1, 10).select('id', 'name', 'age')
        [(3, '张三', 20)]
        """
        if self.return_total:
            sql = get_table_select_sql(self.limit_exec.table, None, self.limit_exec.order_by, self.limit_exec.limit, *columns)
            count_sql = get_table_select_sql(self.limit_exec.table, None, self.limit_exec.order_by, None, SELECT_COUNT)
            return page_select(count_sql, sql, *self.limit_exec.limit)
        return self.limit_exec.select(*columns)

    def query(self, *columns) -> List[dict]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').order_by('id DESC, name ASC').page(1, 10).query('id', 'name', 'age')
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        if self.return_total:
            sql = get_table_select_sql(self.limit_exec.table, None, self.limit_exec.order_by, self.limit_exec.limit, *columns)
            count_sql = get_table_select_sql(self.limit_exec.table, None, self.limit_exec.order_by, None, SELECT_COUNT)
            return page_query(count_sql, sql, *self.limit_exec.limit)
        return self.limit_exec.query(*columns)

    def load(self, *columns) -> Loader:
        """
        Select page data from table and return a Loader instance

        :return: Loader

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').order_by('id DESC, name ASC').page(1, 10).load('id', 'name', 'age')
        Lodder()
        """
        return self.limit_exec.load(*columns)


class ColumnOrderByPageExec:
    
    def __init__(self, order_by_page_exec: OrderByPageExec, *columns):
        self._exec = order_by_page_exec
        self.columns = columns
    
    def select(self) -> List[Tuple]:
        """
        Select data from table and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').page(1, 10).select()
        [(3, '张三', 20)]
        """
        return self._exec.select(*self.columns)
    
    def query(self) -> List[dict]:
        """
        Select data from table and return list results(dict).
        
        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').page(1, 10).query()
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        return self._exec.query(*self.columns)
    
    def to_df(self):
        """
        Select from table and return pandas DataFrame instance.
        
        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').page(1, 10).to_df()
        """
        return self._exec.load(*self.columns).to_df()
    
    def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
        """
        Select from table and sava as a csv file.
    
        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').page(1, 10).to_csv('test.csv')
        """
        self._exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)
    
    def to_json(self, file_name: str, encoding='utf-8'):
        """
        Select from table and sava as a json file.
        
        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').page(1, 10).to_json('test.json')
        """
        self._exec.load(*self.columns).to_json(file_name, encoding)


class WhereOrderByPageExec:
    
    def __init__(self, where_limit_exec: WhereLimitExec, return_total: bool):
        self._exec = where_limit_exec
        self.return_total = return_total
    
    def select(self, *columns) -> List[Tuple]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).select('id', 'name', 'age')
        [(3, '张三', 20)]
        """
        if self.return_total:
            count_sql, sql, args = self._get_page_count_sql_args(*columns)
            return page_select(count_sql, sql, *args, *self._exec.limit_)
        return self._exec.select(*columns)
    
    def query(self, *columns) -> List[dict]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).query('id', 'name', 'age')
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        if self.return_total:
            count_sql, sql, args = self._get_page_count_sql_args(*columns)
            return page_query(count_sql, sql, *args, *self._exec.limit_)
        return self._exec.query(*columns)
    
    def load(self, *columns) -> Loader:
        """
        Select page data from table and return a Loader instance

        :return: Loader

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).load('id', 'name', 'age')
        Lodder()
        """
        return self._exec.load(*columns)
    
    def _get_page_count_sql_args(self, *columns):
        where, args, _ = get_where_arg_limit(**self._exec.where_condition)
        count_sql = get_table_select_sql(self._exec.table, where, None, None, SELECT_COUNT)
        sql = get_table_select_sql(self._exec.table, where, self._exec.order_by_, self._exec.limit_, *columns)
        return count_sql, sql, args


class ColumnWhereOrderByPageExec:
    
    def __init__(self, where_order_by_page_exec: WhereOrderByPageExec, *columns):
        self._exec = where_order_by_page_exec
        self.columns = columns
    
    def select(self) -> List[Tuple]:
        """
        Select data from table and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).select()
        [(3, '张三', 20)]
        """
        return self._exec.select(*self.columns)
    
    def query(self) -> List[dict]:
        """
        Select data from table and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).query()
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        return self._exec.query(*self.columns)
    
    def to_df(self):
        """
        Select from table and return pandas DataFrame instance.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).to_df()
        """
        return self._exec.load(*self.columns).to_df()
    
    def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
        """
        Select from table and sava as a csv file.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).to_csv('test.csv')
        """
        self._exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)
    
    def to_json(self, file_name: str, encoding='utf-8'):
        """
        Select from table and sava as a json file.

        Examples
        --------
        >>> import sqlexecx as db
        >>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10).to_json('test.json')
        """
        self._exec.load(*self.columns).to_json(file_name, encoding)





# !/usr/bin/env python3
# -*- coding:utf-8 -*-

from typing import List, Tuple
from . import exec


class PageExec:

    def __init__(self, _exec, page_num, page_size):
        self.exec = _exec
        self.page_num = page_num
        self.page_size = page_size

    def query(self, sql: str, *args, **kwargs) -> List[dict]:
        """
        Execute select SQL and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.page(1, 10).query('SELECT id, name, age FROM person WHERE name=? and age=?', '张三', 20)
        [{'id': 3, 'name': '张三', 'age': 20}]
        >>> db.page(1, 10).query('SELECT id, name, age FROM person WHERE name=:name and age=:age', name='张三', age=20)
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        """
        Execute select SQL and return list or empty list if no result. Automatically add 'limit ?,?' after sql statement if not.
        sql: SELECT * FROM user WHERE name=? and age=?  -->  args: ('张三', 20)
             SELECT * FROM user WHERE name=:name and age=:age  -->  kwargs: ('张三', 20) --> kwargs: {'name': '张三', 'age': 20}
        """
        return self.exec.query_page(sql, self.page_num, self.page_size, *args, **kwargs)

    def select(self, sql: str, *args, **kwargs) -> List[Tuple]:
        """
        Execute select SQL and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.page(1, 10).select('SELECT id, name, age FROM person WHERE name=? and age=?', '张三', 20)
        [(3, '张三', 20)]
        >>> db.page(1, 10).select('SELECT id, name, age FROM person WHERE name=:name and age=:age', name='张三', age=20)
        [(3, '张三', 20)]
        """
        return self.exec.select_page(sql, self.page_num, self.page_size, *args, **kwargs)

    def do_query(self, sql: str, *args) -> List[dict]:
        """
        Execute select SQL and return list results(dict).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.page(1, 10).do_query('SELECT id, name, age FROM person WHERE name=? and age=?', '张三', 20)
        [{'id': 3, 'name': '张三', 'age': 20}]
        """
        return self.exec.do_query_page(sql, self.page_num, self.page_size, *args)

    def do_select(self, sql: str, *args) -> List[Tuple]:
        """
        Execute select SQL and return list results(tuple).

        Examples
        --------
        >>> import sqlexecx as db
        >>> db.page(1, 10).do_select('SELECT id, name, age FROM person WHERE name=? and age=? LIMIT 1', '张三', 20)
        (3, '张三', 20)
        """
        return self.exec.do_select_page(sql, self.page_num, self.page_size, *args)


def page(page_num=1, page_size=10) -> PageExec:
    """
    Get a PageExec instance

    Examples
    --------
    >>> import sqlexecx as db
    >>> db.page(1, 10)
    """
    return PageExec(exec, page_num, page_size)



from typing import Tuple, List, Union
from .loader import Loader
from .table_support import get_table_select_sql, WhereBase


class LimitExec:
	
	def __init__(self, _exec, table_name: str, order_by: str, limit: Union[int, Tuple[int], List[int]]):
		self.exec = _exec
		self.table = table_name
		self.order_by = order_by
		self.limit = limit
	
	def select(self, *columns) -> List[Tuple]:
		"""
		Select data from table and return list results(tuple).

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').limit(1).select('id', 'name', 'age')
		[(3, '张三', 20)]
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, self.limit, *columns)
		if isinstance(self.limit, int):
			return self.exec.do_select(sql, self.limit)
		return self.exec.do_select(sql, *self.limit)
	
	def query(self, *columns) -> List[Tuple]:
		"""
		Select data from table and return list results(dict).

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').limit(1).query('id', 'name', 'age')
		[{'id': 3, 'name': '张三', 'age': 20}]
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, self.limit, *columns)
		if isinstance(self.limit, int):
			return self.exec.do_query(sql, self.limit)
		else:
			return self.exec.do_query(sql, *self.limit)
	
	def ravel_list(self, column: str) -> List:
		"""
		Select data from table and return list results.

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').limit(3).ravel_list('name')
		['张三', '李四', '王五']
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, self.limit, column)
		if isinstance(self.limit, int):
			return self.exec.do_ravel_list(sql, self.limit)
		return self.exec.do_ravel_list(sql, *self.limit)

	def load(self, *columns) -> Loader:
		"""
		Select page data from table and return a Loader instance
		
		:return: Loader
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').limit(1).load('id', 'name', 'age')
		Lodder()
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, self.limit, *columns)
		if isinstance(self.limit, int):
			return self.exec.do_load(sql, self.limit)
		else:
			return self.exec.do_load(sql, *self.limit)


class ColumnLimitExec:
	
	def __init__(self, limit_exec: LimitExec, *columns):
		self.limit_exec = limit_exec
		self.columns = columns
	
	def select(self) -> List[Tuple]:
		"""
		Select data from table and return list results(tuple).

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').limit(1).select()
		[(3, '张三', 20)]
		"""
		return self.limit_exec.select(*self.columns)
	
	def query(self) -> List[Tuple]:
		"""
		Select data from table and return list results(dict).

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').limit(1).query()
		[{'id': 3, 'name': '张三', 'age': 20}]
		"""
		return self.limit_exec.query(*self.columns)
	
	def ravel_list(self) -> List:
		"""
		Select data from table and return list.

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('name').limit(3).ravel_list()
		['张三', '李四', '王五']
		"""
		return self.limit_exec.ravel_list(*self.columns)
	
	def to_df(self):
		"""
		Select from table and return pandas DataFrame instance.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').limit(10).to_df()
		"""
		return self.limit_exec.load(*self.columns).to_df()

	def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
		"""
		Select from table and sava as a csv file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').limit(10).to_csv('test.csv')
		"""
		self.limit_exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)

	def to_json(self, file_name: str, encoding='utf-8'):
		"""
		Select from table and sava as a json file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').limit(10).to_json('test.json')
		"""
		self.limit_exec.load(*self.columns).to_json(file_name, encoding)
	
	
class WhereLimitExec(WhereBase):

	def __init__(self, _exec, table_name: str, order_by: str, limit: Union[int, Tuple[int], List[int]], **kwargs):
		super().__init__(_exec, table_name, order_by, **kwargs)
		self.limit_ = limit
		
		
class ColumnWhereLimitExec:
	
	def __init__(self, where_limit_exec: WhereLimitExec, *columns):
		self.where_limit_exec = where_limit_exec
		self.columns = columns
	
	def select(self) -> List[Tuple]:
		"""
		Select data from table and return list results(tuple).

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).limit(1).select()
		[(3, '张三', 20)]
		"""
		return self.where_limit_exec.select(*self.columns)

	def query(self) -> List[dict]:
		"""
		Select data from table and return list results(dict).
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).limit(1).query()
		[{'id': 3, 'name': '张三', 'age': 20}]
		"""
		return self.where_limit_exec.query(*self.columns)
	
	def ravel_list(self) -> List:
		"""
		Select data from table and return list.

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('name').where(age=20).limit(3).ravel_list()
		['张三', '李四', '王五']
		"""
		return self.where_limit_exec.ravel_list(*self.columns)

	def to_df(self):
		"""
		Select from table and return pandas DataFrame instance.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).limit(10).to_df()
		"""
		return self.where_limit_exec.load(*self.columns).to_df()

	def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
		"""
		Select from table and sava as a csv file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).limit(10).to_csv('test.csv')
		"""
		self.where_limit_exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)

	def to_json(self, file_name: str, encoding='utf-8'):
		"""
		Select from table and sava as a json file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').where(name='张三', age=20).limit(10).to_json('test.json')
		"""
		self.where_limit_exec.load(*self.columns).to_json(file_name, encoding)



from typing import Tuple, List, Union
from .loader import Loader
from .utils import get_page_start
from .table_support import get_table_select_sql, WhereBase
from .table_limit_exec import LimitExec, ColumnLimitExec, WhereLimitExec, ColumnWhereLimitExec
from .table_page_exec import OrderByPageExec, ColumnOrderByPageExec, WhereOrderByPageExec, ColumnWhereOrderByPageExec


class OrderByExec:
	
	def __init__(self, _exec, table_name: str, order_by: str):
		self.exec = _exec
		self.table = table_name
		self.order_by = order_by

	def select(self, *columns) -> List[Tuple]:
		"""
		Select data from table and return list results(tuple).
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').order_by('id DESC, name ASC').select('id', 'name', 'age')
		[(3, '张三', 20)]
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, None, *columns)
		return self.exec.do_select(sql)
	
	def query(self, *columns) -> List[dict]:
		"""
		Select data from table and return list results(dict).
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').order_by('id DESC, name ASC').query('id', 'name', 'age')
		[{'id': 3, 'name': '张三', 'age': 20}]
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, None, *columns)
		return self.exec.do_query(sql)

	def ravel_list(self, column: str) -> List:
		"""
		Select data from table and return list.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').order_by('id DESC, name ASC').ravel_list('name')
		['张三', '李四', '王五']
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, None, column)
		return self.exec.do_ravel_list(sql)

	def limit(self, limit: Union[int, Tuple[int], List[int]] = 10) -> LimitExec:
		"""
		Get a LimitExec instance
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').order_by('id DESC, name ASC').limit(10)
		LimitExec()
		"""
		return LimitExec(self.exec, self.table, self.order_by, limit)
	
	def page(self, page_num=1, page_size=10, return_total=False) -> OrderByPageExec:
		"""
		Get a OrderByPageExec instance

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').order_by('id DESC, name ASC').page(1, 10)
		OrderByPageExec()
		"""
		return OrderByPageExec(self.limit(limit=(get_page_start(page_num, page_size), page_size)), return_total)

	def load(self, *columns) -> Loader:
		"""
		Select page data from table and return a Loader instance
		
		:return: Loader
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').limit(1).load('id', 'name', 'age')
		Lodder()
		"""
		sql = get_table_select_sql(self.table, None, self.order_by, None, *columns)
		return self.exec.do_load(sql)


class ColumnOrderByExec:
	
	def __init__(self, order_by_exec, *columns):
		self.order_by_exec = order_by_exec
		self.columns = columns

	def select(self) -> List[Tuple]:
		"""
		Select data from table and return list results(tuple).
	
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').select()
		[(3, '张三', 20)]
		"""
		return self.order_by_exec.select(*self.columns)

	def query(self) -> List[dict]:
		"""
		Select data from table and return list results(dict).
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').query()
		[{'id': 3, 'name': '张三', 'age': 20}]
		"""
		return self.order_by_exec.query(*self.columns)

	def ravel_list(self) -> List:
		"""
		Select data from table and return list.
	
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('name').order_by('id DESC, name ASC').ravel_list()
		['张三', '李四', '王五']
		"""
		return self.order_by_exec.ravel_list(*self.columns)

	def limit(self, limit: Union[int, Tuple[int], List[int]] = 10) -> ColumnLimitExec:
		"""
		Get a ColumnLimitExec instance
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').limit(10)
		ColumnLimitExec()
		"""
		return ColumnLimitExec(
			LimitExec(self.order_by_exec.exec, self.order_by_exec.table, self.order_by_exec.order_by, limit),
			*self.columns
		)
	
	def page(self, page_num=1, page_size=10, return_total=False) -> ColumnOrderByPageExec:
		"""
		Get a ColumnOrderByPageExec instance

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').order_by('id DESC, name ASC').page(1, 10)
		ColumnOrderByPageExec()
		"""
		return ColumnOrderByPageExec(self.limit(limit=(get_page_start(page_num, page_size), page_size)), return_total)

	def to_df(self):
		"""
		Select from table and return pandas DataFrame instance.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').limit(10).to_df()
		"""
		return self.order_by_exec.load(*self.columns).to_df()

	def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
		"""
		Select from table and sava as a csv file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').limit(10).to_csv('test.csv')
		"""
		self.order_by_exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)

	def to_json(self, file_name: str, encoding='utf-8'):
		"""
		Select from table and sava as a json file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').limit(10).to_json('test.json')
		"""
		self.order_by_exec.load(*self.columns).to_json(file_name, encoding)


class WhereOrderByExec(WhereBase):
	
	def __init__(self, _exec, table_name: str, order_by: str, **kwargs):
		super().__init__(_exec, table_name, order_by, **kwargs)
	
	def limit(self, limit: Union[int, Tuple[int], List[int]] = 10) -> WhereLimitExec:
		"""
		Get a WhereLimitExec instance

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').where(name='张三', age=20).order_by('id DESC, name ASC').limit(10)
		WhereLimitExec()
		"""
		return WhereLimitExec(self.exec, self.table, self.order_by_, limit, **self.where_condition)
	
	def page(self, page_num=1, page_size=10, return_total=False) -> WhereOrderByPageExec:
		"""
		Get a WhereOrderByPageExec instance

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10)
		WhereOrderByPageExec()
		"""
		return WhereOrderByPageExec(self.limit(limit=(get_page_start(page_num, page_size), page_size)), return_total)
	
	
class ColumnWhereOrderByExec:
	
	def __init__(self, where_order_by_exec: WhereOrderByExec, *columns):
		self.where_order_by_exec = where_order_by_exec
		self.columns = columns
	
	def select(self) -> List[Tuple]:
		"""
		Select data from table and return list results(tuple).

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').select()
		[(3, '张三', 20)]
		"""
		return self.where_order_by_exec.select(*self.columns)

	def query(self) -> List[dict]:
		"""
		Select data from table and return list results(dict).
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').query()
		[{'id': 3, 'name': '张三', 'age': 20}]
		"""
		return self.where_order_by_exec.query(*self.columns)

	def limit(self, limit: Union[int, Tuple[int], List[int]] = 10) -> ColumnWhereLimitExec:
		"""
		Get a ColumnWhereLimitExec instance
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').limit(10)
		ColumnWhereLimitExec()
		"""
		return ColumnWhereLimitExec(self.where_order_by_exec.limit(limit), *self.columns)
	
	def ravel_list(self) -> List:
		"""
		Select data from table and return list.

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('name').where(name='张三', age=20).order_by('id DESC, name ASC').ravel_list()
		['张三', '李四', '王五']
		"""
		return self.where_order_by_exec.ravel_list(*self.columns)
	
	def page(self, page_num=1, page_size=10, return_total=False) -> ColumnWhereOrderByPageExec:
		"""
		Get a ColumnWhereOrderByPageExec instance

		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').where(name='张三', age=20).order_by('id DESC, name ASC').page(1, 10)
		ColumnWhereOrderByPageExec()
		"""
		return ColumnWhereOrderByPageExec(
			WhereOrderByPageExec(self.where_order_by_exec.limit((get_page_start(page_num, page_size), page_size)), return_total),
			*self.columns
		)

	def to_df(self):
		"""
		Select from table and return pandas DataFrame instance.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db.table('person').columns('id', 'name', 'age').limit(10).to_df()
		"""
		return self.where_order_by_exec.load(*self.columns).to_df()

	def to_csv(self, file_name: str, delimiter=',', header=True, encoding='utf-8'):
		"""
		Select from table and sava as a csv file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').limit(10).to_csv('test.csv')
		"""
		self.where_order_by_exec.load(*self.columns).to_csv(file_name, delimiter, header, encoding)

	def to_json(self, file_name: str, encoding='utf-8'):
		"""
		Select from table and sava as a json file.
		
		Examples
		--------
		>>> import sqlexecx as db
		>>> db..table('person').columns('id', 'name', 'age').limit(10).to_json('test.json')
		"""
		self.where_order_by_exec.load(*self.columns).to_json(file_name, encoding)



"""
Examples
--------
>>> import sqlexecx as db
>>> db.init('db.sqlite3', driver='sqlite3', show_sql=True, debug=True)
Engine.SQLITE
>>> sql = 'insert into person(name, age) values(%s, %s)'
>>> db.execute(sql, '张三', 20)
1
>>> sql = 'SELECT id, name, age FROM person WHERE name=%s and age=%s'
>>> db.select(sql, '张三', 20)
[(3, '张三', 20)]
>>> db.select_one(sql, '张三', 20)
(3, '张三', 20)
>>> db.query(sql, '张三', 20)
[{'id': 3, 'name': '张三', 'age': 20}]
>>> db.query_one(sql, '张三', 20)
{'id': 3, 'name': '张三', 'age': 20}
>>> sql = 'SELECT count(1) FROM person WHERE name=%s and age=%s LIMIT 1'
>>> db.get(sql, '张三', 20)
1
"""

from sqlexecutorx import (
    conn,
    trans,
    get_connection,
    close,
    Driver,
    Engine,
    InitArgs,
    init as _init
)
from .exec import (
    execute,
    insert,
    save,
    save_sql,
    save_select_key,
    save_sql_select_key,
    batch_insert,
    batch_execute,
    get,
    select,
    select_one,
    query,
    query_one,
    ravel_list,
    select_page,
    query_page,
    load,
    do_execute,
    do_save_sql,
    do_save_sql_select_key,
    do_get,
    do_select,
    do_select_one,
    do_query,
    do_query_one,
    do_ravel_list,
    do_select_page,
    do_query_page,
    do_load,
    insert_from_df,
    insert_from_csv,
    insert_from_json,
    truncate_table,
    drop_table,
    show_tables
)

from .sql_exec import sql
from .page_exec import page
from .dialect import Dialect
from .table_exec import table


def init(*args, **kwargs) -> Engine:
    """
    Compliant with the Python DB API 2.0 (PEP-249).

    Addition parameters:
    :param driver=None: str|Driver, import driver, 'import pymysql'
    :param pool_size=0: int, default 0, size of connection pool
    :param show_sql=False: bool,  if True, print sql
    :param debug=False: bool, if True, print debug context

    Other parameters of connection pool refer to DBUtils: https://webwareforpython.github.io/DBUtils/main.html#pooleddb-pooled-db

    Examples
    --------
    >>> import sqlexecx as db
    >>> db.init('db.sqlite3', driver='sqlite3', debug=True)
    >>> or
    >>> db.init("postgres://user:password@127.0.0.1:5432/testdb", driver='psycopg2', pool_size=5, debug=True)
    >>> or
    >>> db.init(user='root', password='xxx', host='127.0.0.1', port=3306, database='testdb', driver='pymysql')
    """

    engine = _init(*args, **kwargs)
    Dialect.init(engine)
    return engine



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

import re
from functools import lru_cache
from typing import Sequence, Collection, Union, Optional
from sqlexecutorx.sql_support import require_limit
from sqlexecutorx import execute, get, query, select, DBError, Engine
from .log_support import logger
from .constant import CACHE_SIZE, LIMIT_1, DEFAULT_KEY_FIELD, MYSQL_COLUMN_SQL, MYSQL_SELECT_KEY, POSTGRES_COLUMN_SQL,\
    SQLITE_SELECT_KEY
from . import utils

NAMED_REGEX = r"like\s+'[\w|\d|%]*'"
PATTERN = re.compile(NAMED_REGEX, re.I)


def handle_like(sql: str):
    for r in re.finditer(PATTERN, sql):
        old = r.group()
        new = old.replace('%', '%%').replace('%%%%', '%%')
        sql = sql.replace(old, new, 1)
    return sql


class BaseDialect:

    def __init__(self, engine: Engine):
        self.engine = engine

    @staticmethod
    def create_insert_sql(table_name: str, cols: Collection[str], placeholder: str = '%s') -> str:
        columns, placeholders = zip(*[('{}'.format(col), placeholder) for col in cols])
        return f"INSERT INTO {table_name}({', '.join(columns)}) VALUES({','.join(placeholders)})"

    @staticmethod
    def before_execute(sql: str) -> str:
        if '%' in sql and 'like' in sql.lower():
            sql = handle_like(sql)
        return sql.replace('?', '%s')

    @staticmethod
    def get_page_sql_args(sql: str, page_num: int, page_size: int, *args):
        start = utils.get_page_start(page_num, page_size)
        if require_limit(sql):
            sql = BaseDialect.limit_offset_sql(sql)
        args = [*args, page_size, start]
        return sql, args

    @staticmethod
    def limit_offset_sql(sql: str):
        return f'{sql} LIMIT ? OFFSET ?'

    @staticmethod
    def get_table_columns(table_name: str) -> str:
        return '*'

    @staticmethod
    def truncate_table(table_name: str):
        return execute(f'TRUNCATE TABLE {Dialect.get_dialect_str(table_name)}')

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def get_dialect_str(src: Union[str, int, Collection[str]]) -> str:
        assert isinstance(src, str), 'source must be a string.'
        return src.strip() if isinstance(src, str) else ','.join([s.strip() for s in src])

    @staticmethod
    def get_select_key(*args, **kwargs):
        raise NotImplementedError("Not implement method 'get_select_key', you can use sqlormx snowflake for primary key.")

    @staticmethod
    def show_tables(schema: Optional[str]):
        return NotImplementedError


class MySQLDialect(BaseDialect):

    @staticmethod
    def create_insert_sql(table_name: str, cols: Sequence[str], placeholder: str = '%s') -> str:
        columns, placeholders = zip(*[('`{}`'.format(col), placeholder) for col in cols])
        return f"INSERT INTO `{table_name}`({','.join(columns)}) VALUES({','.join(placeholders)})"

    @staticmethod
    def get_page_sql_args(sql: str, page_num: int, page_size: int, *args):
        start = utils.get_page_start(page_num, page_size)
        if require_limit(sql):
            sql = MySQLDialect.limit_offset_sql(sql)
        args = [*args, start, page_size]
        return sql, args

    @staticmethod
    def limit_offset_sql(sql: str):
        return f'{sql} LIMIT ?, ?'

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def get_table_columns(table_name: str) -> str:
        return get(MYSQL_COLUMN_SQL, table_name, LIMIT_1)

    @staticmethod
    def get_select_key(*args, **kwargs) -> str:
        return MYSQL_SELECT_KEY

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def get_dialect_str(src: Union[str, int, Collection[str]]) -> str:
        assert src, 'src is required.'
        if isinstance(src, str):
            return _dialect_str(src)
        return ','.join([_dialect_str(arg) for arg in src])

    @staticmethod
    def show_tables(schema: Optional[str]):
        if schema:
            execute(f'use {schema}')
        return [table[0] for table in select('show tables')]


class PostgresDialect(BaseDialect):

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def get_table_columns(table_name: str) -> str:
        return get(POSTGRES_COLUMN_SQL, table_name, LIMIT_1)

    @staticmethod
    def get_select_key(key_seq: str = None, table_name: str = None, key: str =None, sql: str = None) -> str:
        if not key_seq:
            if table_name:
                key_seq = PostgresDialect.build_key_seq(table_name, key)
            else:
                if sql:
                    key_seq = PostgresDialect._get_key_seq_from_sql(sql)
                else:
                    raise DBError("Get PostgreSQL select key fail, all of 'key_seq', 'table', 'sql' are None")
        return f"SELECT currval('{key_seq}')"

    @staticmethod
    def build_key_seq(table_name: str, key: str = None) -> str:
        if not key:
            key = DEFAULT_KEY_FIELD
        return f'{table_name}_{key}_seq'

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def _get_key_seq_from_sql(sql: str) -> str:
        table_name = re.search('(?<=into )\w+', sql, re.I)
        key_seq = PostgresDialect.build_key_seq(table_name.group())
        logger.warning("'key_seq' is None, will use default '{}' from sql.".format(key_seq))
        return key_seq

    @staticmethod
    def show_tables(schema: Optional[str]):
        sql = '''
        SELECT tablename FROM pg_catalog.pg_tables  
         WHERE schemaname != 'pg_catalog'  
           AND schemaname != 'information_schema'
       '''
        if schema:
            execute(f'use {schema}')
        return [table[0] for table in select(sql)]


class OracleDialect(BaseDialect):

    @staticmethod
    def get_page_sql_args(sql: str, page_num: int, page_size: int, *args):
        start = utils.get_page_start(page_num, page_size)
        end = start + page_size
        sql = f'SELECT * FROM (SELECT tmp.*, rownum row_num FROM ({sql}) tmp WHERE rownum <= ?) WHERE row_num > ? '
        args = [*args, end, start]
        return sql, args

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def get_table_columns(table_name: str) -> str:
        sql = 'SELECT column_name FROM user_tab_columns WHERE table_name = %s'
        results = select(sql, table_name)
        return ','.join([result[0] for result in results])


class SQLiteDialect(MySQLDialect):
    
    @staticmethod
    def create_insert_sql(table_name: str, cols: Sequence[str], placeholder: str = '?') -> str:
        return MySQLDialect.create_insert_sql(table_name, cols, placeholder)

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def get_table_columns(table_name: str) -> str:
        results = query(f'PRAGMA table_info(`{table_name}`)')
        return ','.join([f"`{result['name']}`" for result in results])

    @staticmethod
    def get_select_key(*args, **kwargs) -> str:
        return SQLITE_SELECT_KEY

    @staticmethod
    def before_execute(sql: str) -> str:
        if '%' in sql and 'like' in sql.lower():
            sql = handle_like(sql)
        return sql

    @staticmethod
    def truncate_table(table_name: str) -> int:
        return execute(f'DELETE FROM `{table_name}`')

    @staticmethod
    def show_tables(schema: Optional[str]):
        return [table[0] for table in select("SELECT name FROM sqlite_master WHERE type='table'")]
    

_DIALECT = None


class Dialect:

    @classmethod
    def init(cls, engine: Engine):
        global _DIALECT
        if _DIALECT is None:
            if Engine.MYSQL == engine:
                _DIALECT = MySQLDialect(engine)
            elif Engine.POSTGRESQL == engine:
                _DIALECT = PostgresDialect(engine)
            elif Engine.ORACLE == engine:
                _DIALECT = OracleDialect(engine)
            elif Engine.SQLITE == engine:
                _DIALECT = SQLiteDialect(engine)
            else:
                _DIALECT = BaseDialect(engine)

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def create_insert_sql(table_name: str, cols: Collection[str]) -> str:
        return _DIALECT.create_insert_sql(table_name, cols)

    @staticmethod
    @lru_cache(maxsize=CACHE_SIZE)
    def before_execute(sql: str) -> str:
        return _DIALECT.before_execute(sql)

    @staticmethod
    def get_page_sql_args(sql: str, page_num: int, page_size: int, *args):
        return _DIALECT.get_page_sql_args(sql, page_num, page_size, *args)
    
    @staticmethod
    def limit_offset_sql(sql: str):
        return _DIALECT.limit_offset_sql(sql)

    @staticmethod
    def get_table_columns(table_name: str) -> str:
        return _DIALECT.get_table_columns(table_name)

    @staticmethod
    def truncate_table(table_name: str) -> int:
        return _DIALECT.truncate_table(table_name)

    @staticmethod
    def get_select_key(*args, **kwargs) -> str:
        return _DIALECT.get_select_key(*args, **kwargs)

    @staticmethod
    def get_dialect_str(src: Union[str, int, Collection[str]]) -> str:
        return _DIALECT.get_dialect_str(src)

    @staticmethod
    def curr_engine() -> Engine:
        return _DIALECT.engine

    @staticmethod
    def show_tables(schema: Optional[str]):
        return _DIALECT.show_tables(schema)


def _dialect_str(src: Union[str, int]):
    assert isinstance(src, str) or isinstance(src, int), 'source must be a string or int.'
    if isinstance(src, int):
        return str(src)
    elif '(' in src or ' as ' in src.lower():
        return src.strip()
    elif ',' in src:
        return ','.join([f"`{s.strip()}`"for s in src.split(',')])
    else:
        return f'`{src.strip()}`'



from sqlexecutorx.log_support import logger, do_sql_log, do_save_log, batch_sql_log, db_ctx_log
from .constant import MODULE


def insert_log(function: str, table: str, **kwargs):
    logger.debug(f"Exec func '{MODULE}.{function}' \n\t Table: '{table}', kwargs: {kwargs}")


def save_log(function: str, select_key: str, table: str, **kwargs):
    logger.debug(f"Exec func '{MODULE}.{function}', 'select_key': {select_key} \n\t Table: '{table}', kwargs: {kwargs}")


def sql_log(function: str, sql: str, *args, **kwargs):
    logger.debug(f"Exec func '{function}' \n\tsql: {sql.strip()} \n\targs: {args} \n\tkwargs: {kwargs}")


def do_sql_log(function: str, sql: str, *args):
    logger.debug(f"Exec func '{function}' \n\t sql: {sql.strip()} \n\t args: {args}")

#
# def db_ctx_log(action, connection):
#     logger.debug("%s connection <%s>..." % (action, hex(id(connection))))

